package Wrapper;
#
# $Id: Wrapper.pm,v 1.5 2001/09/28 17:48:20 voeckler Exp voeckler $
#
require 5.6.0;
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw();
@EXPORT_OK = qw($VERSION);

$VERSION = '0.02';

# ---------------------------------------------------------------
use DBI;
use Carp;

#
# auxillary functions
#

#
# ctor
#
sub new {
    # purpose: connect to database on object instantiation
    # paramtr: $name (IN): name of database to connect to
    #          $user (IN): username to use for connecting to database
    #          $pass (IN): password to use when connecting user
    # warning: dies with error, if unable to connect to database
    my $proto = shift;
    my $class = ref($proto) || $proto || 'Wrapper';

    # create instance
    my $self = bless { m_handle => undef, m_count => 0, m_force => 0,
		       m_debug => { }, m_transformation => undef }, $class;

    # connect to database or die
    my $name = shift;
    my $user = shift;
    my $pass = shift;
    $name = $user unless defined $name;
    $self->{m_handle} = 
	DBI->connect( "DBI:Pg:dbname=$name", $user, $pass,
		      { PrintError => 0, # warn (no) about error conditions
			RaiseError => 1, # but do (not) die on errors
			AutoCommit => 0, # commit on commit only
			@_ } )	         # additional attributes (overwrite)
	    or croak $DBI::errstr; # complain from view of caller

    # return instance
    $self;
}

#
# dtor
#
sub DESTROY {
    # purpose: disconnect from database, rolling back uncommitted xactions
    my $self = shift;

    if ( $self->{m_count} > 0 ) {
	carp "rolling back ", $self->{m_count}, " modifications!\n";
	$self->{m_handle}->rollback() 
	    or croak $DBI::errstr;
    }
    $self->{m_handle}->disconnect();
}

#
# accessors
#
sub increment_force {
    my $self = shift;
    ++$self->{m_force};
}

sub decrement_force {
    my $self = shift;
    --$self->{m_force};
}

sub force {
    my $self = shift;
    my $oldv = $self->{m_force};
    $self->{m_force} = shift if ( @_ );
    $oldv;
}

sub handle {
    my $self = shift;
    my $oldv = $self->{m_handle};
    $self->{m_handle} = shift if ( @_ );
    $oldv;
}

sub count {
    my $self = shift;
    my $oldv = $self->{m_count};
    $self->{m_count} = shift if ( @_ );
    $oldv;
}

sub debug {
    my $self = shift;
    my $oldv = $self->{m_debug};
    $self->{m_debug} = shift if ( @_ );
    $oldv;
}

# sub xaction {
#     my $self = shift;
#     my $oldv = $self->{m_transaction};
#     $self->{m_transaction} = shift if ( @_ );
#     $oldv;
# }

#
# generic member functions
#
sub commit {
    # purpose: wrapper around commit, resets counter
    # returns: number of open updates committed, or undef in case of error
    my $self = shift;

    my $result;
    eval {
	if ( $self->{m_handle}->commit ) {
	    $result = $self->{m_count} || '0E0';
	    $self->{m_count} = 0;
	}
    };

    $result;
}

sub quote {
    # purpose: short-cut, multi-purpose quote
    # returns: array of quoted arguments for more than one arg
    #          first argument as quoted scalar if zero or one arg
    my $self = shift;
    return ( @_ > 1 ) ?
	map { $self->{m_handle}->quote($_) } @_ :
	$self->{m_handle}->quote( $_[0] || 'null' );
}

sub rollback {
    # purpose: wrapper around rollback, resets counter
    # returns: number of open updates dropped, or undef in case of error
    my $self = shift;

    my $result;
    eval {
	if ( $self->{m_handle}->rollback ) {
	    $result = $self->{m_count} || '0E0';
	    $self->{m_count} = 0;
	}
    };

    $result;
}

sub get_sequence {
    # purpose: obtain current value of sequence
    # paramtr: $seqname (IN): name of sequence
    # returns: value read from sequence, or undef on error (check DBI::errstr)
    # globals: $main::handle
    my $self = shift;
    my $name = shift;

    my @row;
    eval { 
	@row = $self->{m_handle}->selectrow_array( "select nextval(\'$name\')" );
    };
    print( "-- error: nextval($name): $DBI::errstr\n" ) if ( $@ );
    $row[0];
}

sub modify {
    # purpose: wrapper like $DBI->do
    # paramtr: $stmt: statement to prepare and execute
    #          [...]: any number of bind variables referenced in $stmt
    # warning: if $stmt is a reference, the prepare step will be skipped.
    # warning: you need to quote all necessary parameters
    # returns: number of modified rows, or undef in case of error.
    my $self = shift;
    my $stmt = shift;

    my ($sth,$result);
    eval {
	local $SIG{__DIE__};

	# prepare statement
	if ( ref $stmt ) {
	    # statement was already prepared
	    $sth = $stmt;
	    $stmt = $sth->{Statement};
	} else {
	    # statement needs preparing
	    $sth = $self->{m_handle}->prepare($stmt)
		or croak "unable to prepare $stmt: $DBI::errstr\n";
	}
	$stmt =~ s/[ \t\n\r]{2,}/ /g;
	print "-- debug: statement => $stmt\n"
	    if $self->debug->{modify};

	# execute statement
	$result = ( @_ ? $sth->execute( @_ ) : $sth->execute )
	    or croak "unable to execute $stmt: ", $sth->errstr, "\n";
	print "-- debug: args => ", join(',',@_), "\n"
	    if $self->debug->{modify};
	$self->{m_count} += $result;
    };
    print( '-- error: ', $@ ) if ( $@ || (ref $sth && $sth->err) );
    $sth->finish;
    $result;
}

sub fetch {
    # purpose: wrapper like $DBI->selectall_arrayref
    # paramtr: $stmt: statement to prepare and execute
    #          [...]: any number of bind variables referenced in $stmt
    # warning: if $stmt is a reference, the prepare step will be skipped.
    # warning: you need to quote all necessary parameters
    # returns: a reference to a vector of vectors, or undef in case of error.
    my $self = shift;
    my $stmt = shift;

    my ($sth,$result);
    eval {
	local $SIG{__DIE__};

	# prepare statement
	if ( ref $stmt ) {
	    # statement was already prepared
	    $sth = $stmt;
	    $stmt = $sth->{Statement};
	} else {
	    # statement needs preparing
	    $sth = $self->{m_handle}->prepare($stmt)
		or croak "unable to prepare $stmt: $DBI::errstr\n";
	}
	$stmt =~ s/[ \t\n\r]{2,}/ /g;
	print "-- debug: statement => $stmt\n"
	    if $self->debug->{fetch};

	# execute statement
	( @_ ? $sth->execute( @_ ) : $sth->execute )
	    or croak "unable to execute $stmt: ", $sth->errstr, "\n";
	print "-- debug: args => ", join(',',@_), "\n"
	    if $self->debug->{fetch};

	# fetch results
	$result = $sth->fetchall_arrayref
	    or croak "unable to fetch from $stmt: ", $sth->errstr, "\n";
	print "-- debug: ", Data::Dumper->Dump([$result],[qw($result)]), "\n"
	    if $self->debug->{fetch} > 1;
    };
    print( '-- error: ', $@ ) if ( $@ || (ref $sth && $sth->err) );
    $sth->finish;
    $result;
}

#
# specific member functions => new module
#



#
# return 'true' to package loader
#
1;
